package com.example.onlinejudge.controller;


import cn.dev33.satoken.stp.StpUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import com.example.onlinejudge.model.VO.QuestionSubmitVO;
import com.example.onlinejudge.model.dto.questionSubmit.QuestionSubmitAddRequest;
import com.example.onlinejudge.model.dto.questionSubmit.QuestionSubmitQueryRequest;
import com.example.onlinejudge.model.entity.QuestionSubmit;
import com.example.onlinejudge.common.result.Result;
import com.example.onlinejudge.common.result.ResultCode;
import com.example.onlinejudge.service.QuestionSubmitService;
import com.example.onlinejudge.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;


@Api(tags = "提交代码判题")
@RestController
@RequestMapping("/question_submit")
@Slf4j
public class QuestionSubmitController {

    @Resource
    private QuestionSubmitService questionSubmitService;

    @Resource
    private UserService userService;

    @ApiOperation(value = "提交题目")
    @PostMapping("/")
    public Result doQuestionSubmit(@RequestBody QuestionSubmitAddRequest questionSubmitAddRequest) {
        if (questionSubmitAddRequest == null || questionSubmitAddRequest.getQuestionId() <= 0) {
            //题目不存在
            return new Result(ResultCode.PARAM_IS_INVALID.getCode(),ResultCode.PARAM_IS_INVALID.getMessage());
        }
        long logInId = StpUtil.getLoginIdAsLong();
        System.out.println("logInId: " + logInId);
        long questionSubmitId = questionSubmitService.doQuestionSubmit(questionSubmitAddRequest, logInId);
        return Result.success(questionSubmitId);
    }

    @ApiOperation(value = "分页获取题目提交列表（除了管理员外，普通用户只能看到非答案、提交代码等公开信息）")
    @PostMapping("/list/page")
    public Result<Page<QuestionSubmitVO>> listQuestionSubmitByPage(@RequestBody QuestionSubmitQueryRequest questionSubmitQueryRequest) {
        long current = questionSubmitQueryRequest.getCurrent();
        long size = questionSubmitQueryRequest.getPageSize();
        // 从数据库中查询原始的题目提交分页信息
        Page<QuestionSubmit> questionSubmitPage = questionSubmitService.page(new Page<>(current, size),
                questionSubmitService.getQueryWrapper(questionSubmitQueryRequest));
        final long logInId = StpUtil.getLoginIdAsLong();
        return Result.success(questionSubmitService.getQuestionSubmitVOPage(questionSubmitPage, logInId));
    }


}
