package com.example.onlinejudge.controller;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.StrUtil;
import com.example.onlinejudge.model.dto.user.UserLoginRequest;
import com.example.onlinejudge.common.result.Result;
import com.example.onlinejudge.common.result.ResultCode;
import com.example.onlinejudge.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;


@Api(tags = "用户登录")
@Controller
@RequestMapping("/api/user")
public class UserController {

    @Resource
    private UserService userService;

    @ApiOperation(value = "用户登录")
    @PostMapping("/login")
    @ResponseBody
    public Result login(@RequestBody UserLoginRequest userLoginRequest) {
        if(userLoginRequest == null) {
            return Result.error(ResultCode.PARAM_IS_INVALID.getCode(),ResultCode.PARAM_IS_INVALID.getMessage());
        }
        String username = userLoginRequest.getUserName();
        String password = userLoginRequest.getPassword();
        if(StrUtil.isBlank(username) || StrUtil.isBlank(password)) {
            return Result.error(ResultCode.PARAM_IS_BLANK.getCode(),ResultCode.PARAM_IS_BLANK.getMessage());
        }
        return userService.login(username, password);
    }

    @ApiOperation(value = "判断用户是否登录")
    @GetMapping("/isLogin")
    @ResponseBody
    public String isLogin() {
        return "当前会话登录了";
    }

    @ApiOperation(value = "用户登出")
    @GetMapping("/logout")
    @ResponseBody
    public Result logout() {
        StpUtil.logout();
        return Result.success();
    }

}
